"""
Registry functions package for VEX AST.

This package contains the function definitions for the VEX API registry.
"""

from .initialize import initialize_registry

__all__ = [
    "initialize_registry"
]